/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

define(
    [
        'jquery',
        'underscore',
        'ko',
        'Magento_Ui/js/form/form',
        'uiRegistry',
        'Magento_Checkout/js/model/quote',
        'Aheadworks_OneStepCheckout/js/model/payment-actions',
        'Magento_Checkout/js/model/full-screen-loader',
        'Magento_Checkout/js/action/select-shipping-address',
        'Magento_Checkout/js/action/select-billing-address',
        'Aheadworks_OneStepCheckout/js/action/set-shipping-information',
        'Aheadworks_OneStepCheckout/js/model/same-as-shipping-flag',
        'Aheadworks_OneStepCheckout/js/model/place-order-allowed-flag',
        'mage/translate',
        'Magento_Checkout/js/model/payment/additional-validators'
    ],
    function (
        $,
        _,
        ko,
        Component,
        registry,
        quote,
        paymentActionsList,
        fullScreenLoader,
        selectShippingAddressAction,
        selectBillingAddressAction,
        setShippingInformationAction,
        sameAsShippingFlag,
        placeOrderAllowedFlag,
        $t,
        additionalValidators
    ) {
        'use strict';

        return Component.extend({
            defaults: {
                template: 'Aheadworks_OneStepCheckout/place-order'
            },
            isPlaceOrderActionAllowed: placeOrderAllowedFlag,
            placeOrderActionTitle: ko.observable($t('Place Order')),

            /**
             * @inheritdoc
             */
            initialize: function () {
                var self = this;

                this._super();

                quote.paymentMethod.subscribe(function (methodData) {
                    if (methodData) {
                        self._setPlaceOrderActionTitle(methodData.method);
                    }
                });
                paymentActionsList.getItems().subscribe(function () {
                    if (quote.paymentMethod()) {
                        self._setPlaceOrderActionTitle(quote.paymentMethod().method);
                    }
                });

                return this;
            },

            /**
             * Perform checkout data validation
             *
             * @returns {Boolean}
             */
            _validate: function () {
                var isValid = true;

                if (!this._validateAddresses()) {
                    isValid = false;
                }
                if (!this._validateShippingMethod()) {
                    isValid = false;
                }
                if (!this._validateDeliveryDateFormData()) {
                    isValid = false;
                }
                if (!this._validatePaymentMethod()) {
                    isValid = false;
                }

                return isValid;
            },

            /**
             * Validate addresses data
             *
             * @returns {Boolean}
             */
            _validateAddresses: function () {
                var isValid = true;

                _.each(['checkout.shippingAddress', 'checkout.paymentMethod.billingAddress'], function (query) {
                    var addressComponent = registry.get(query);

                    addressComponent.validate();
                    if (this.source.get('params.invalid')) {
                        isValid = false;
                    }
                }, this);

                return isValid;
            },

            /**
             * Validate shipping method
             *
             * @returns {boolean}
             */
            _validateShippingMethod: function () {
                var shippingMethodComponent = registry.get('checkout.shippingMethod');

                shippingMethodComponent.validate();

                return !this.source.get('params.invalid');
            },

            /**
             * Validate delivery date form data
             *
             * @returns {boolean}
             */
            _validateDeliveryDateFormData: function () {
                var deliveryDateComponent = registry.get('checkout.shippingMethod.delivery-date');

                deliveryDateComponent.validate();

                return !this.source.get('params.invalid');
            },

            /**
             * Validate payment method
             *
             * @returns {boolean}
             */
            _validatePaymentMethod: function () {
                var methodListComponent = registry.get('checkout.paymentMethod.methodList'),
                    methodRenderer,
                    validateMethod;

                if (quote.paymentMethod()) {
                    methodRenderer = methodListComponent.getChild(quote.paymentMethod().method);
                    validateMethod = 'validateHandler' in methodRenderer
                        ? 'validateHandler'
                        : 'validate';

                    return methodListComponent.validate()
                        && methodRenderer[validateMethod]()
                        && additionalValidators.validate();
                } else {
                    return methodListComponent.validate();
                }
            },

            /**
             * Set place order action title
             *
             * @param {String} paymentMethodCode
             * @returns {String}
             */
            _setPlaceOrderActionTitle: function (paymentMethodCode) {
                var action = paymentActionsList.get(paymentMethodCode);

                if (action) {
                    this.placeOrderActionTitle(action.find('span').text());
                }
            },

            /**
             * On place order button click event handler
             *
             * @param {Object} data
             * @param {Event} event
             */
            onPlaceOrderClick: function (data, event) {
                var self = this;

                if (event) {
                    event.preventDefault();
                }
                if (this.isPlaceOrderActionAllowed() && this._validate()) {
                    fullScreenLoader.startLoader();
                    this._setShippingInformation().done(
                        function () {
                            self._setPaymentInformation().done(function () {
                                var paymentAction = paymentActionsList.get(quote.paymentMethod().method);

                                fullScreenLoader.stopLoader();
                                paymentAction.trigger('click');
                            });
                        }
                    );
                }
            },

            /**
             * Set shipping information
             *
             * @returns {Object}
             */
            _setShippingInformation: function () {
                var shippingAddressComponent = registry.get('checkout.shippingAddress'),
                    shippingAddress = quote.shippingAddress();

                if (!quote.isQuoteVirtual()) {
                    if (shippingAddressComponent.useFormData()) {
                        shippingAddress = shippingAddressComponent.copyFormDataToQuoteData(shippingAddress);
                    }
                    selectShippingAddressAction(shippingAddress);

                    return setShippingInformationAction();
                }
                return $.Deferred().resolve();
            },

            /**
             * Set payment information
             *
             * @returns {Object}
             */
            _setPaymentInformation: function () {
                var billingAddressComponent = registry.get('checkout.paymentMethod.billingAddress'),
                    billingAddress = quote.billingAddress()
                        ? quote.billingAddress()
                        : quote.shippingAddress();

                if (billingAddressComponent.useFormData()) {
                    billingAddress = billingAddressComponent.copyFormDataToQuoteData(billingAddress);
                } else if (!quote.isQuoteVirtual() && sameAsShippingFlag.sameAsShipping()) {
                    billingAddress = quote.shippingAddress()
                }
                selectBillingAddressAction(billingAddress);

                return $.Deferred().resolve();
            }
        });
    }
);
